/******************************************************************************
    (c) 1998 P.J. Caulfield               patrick@pandh.demon.co.uk
                                          pcaulfield@cix.co.uk
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ******************************************************************************
*/

#include <sys/types.h>
#include <sys/socket.h>
#include <stdio.h>
#include <syslog.h>
#include <ctype.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>
#include <netdnet/dnetdb.h>
#include <netdnet/dn.h>
#include "configfile.h"

#ifndef SENDMAIL_COMMAND
#define SENDMAIL_COMMAND "/usr/sbin/sendmail -oem"
#endif

#define NONAME_ERROR "The user does not exist on this system"

extern int verbosity;

// Receive a message from VMS and pipe it into sendmail
void receive_mail(int sock)
{
    char  remote_user[256]; // VMS only sends 12 but...just in case!
    char  local_user[256];
    char  full_user[256];
    char  subject[256];
    char  remote_hostname[256];
    struct sockaddr_dn sockaddr;
    char  buf[1024];
    FILE *mailpipe;
    int   stat;
    int   i;
    int   namlen = sizeof(sockaddr);

    // Get the remote host name
    stat = getpeername(sock, (struct sockaddr *)&sockaddr, &namlen);
    if (!stat)
    {
        strcpy(remote_hostname, dnet_htoa(&sockaddr.sdn_add));
    }
    else
    {
	sprintf(remote_hostname, "%d.%d", sockaddr.sdn_add.a_addr[0],
		sockaddr.sdn_add.a_addr[0]); 
    }
    
    // Read the remote user name - this comes in padded with spaces to
    // 12 characters
    stat = read(sock, remote_user, 12); // PJC change this when Eduardo adds my "packet patch" to the kernel
    if (stat < 0)
    {
	perror("Error reading remote user");
	return;
    }

    // Trim the remote user.
    remote_user[stat] = '\0'; // Just in case its the full buffer
    i=stat;
    while (remote_user[i] == ' ') remote_user[i--] = '\0';
    
    // The rest of the message should be the local user name. This could
    // be a real local user or an internet mail name. It is not
    // padded.
    stat = read(sock, local_user, sizeof(local_user));
    if (stat == -1)
    {
	perror("Error reading local user");
	return;
    }
    local_user[stat] = '\0';
    for (i=0; i<stat; i++) local_user[i] = tolower(local_user[i]);

    // Do a quick check on the user. If it has an @ it it then just let it
    // through, otherwise see if it exists on this machine.
#if 0
    if (!strchr(local_user, '@'))
    {
	struct passwd *p = getpwnam(local_user);
	
	if (!p) // Never 'eard of 'im.
	{
	    char failmsg[] = { 0x12, 0x81, 0x7e, 0x00 };
	    write(sock, failmsg, sizeof(failmsg));
	    write(sock, NONAME_ERROR, strlen(NONAME_ERROR));
	    write(sock, "\0", 1);
	    return;
	}
    }
#endif
    
    // Send acknowledge
    write(sock, "\001\000\000\000", 4);

    // mail sends a single NUL now
    stat = read(sock, buf, 1);
    if (stat == -1)
    {
	perror("Error reading ");
	return;
    }

    // Not sure what the relationship is between this and the "local_user"
    // we got earlier. This one has the double-colon in it though!
    stat = read(sock, full_user, sizeof(full_user));
    if (stat == -1)
    {
	perror("Error reading full_user");
	return;
    }
    full_user[stat] = '\0';
    
    // Get the subject
    stat = read(sock, subject, sizeof(subject));
    if (stat == -1)
    {
	perror("Error reading subject");
	return;
    }
    subject[stat] = '\0';


    syslog(LOG_INFO, "Forwarding mail from %s::%s to %s\n",
	   remote_hostname, remote_user, local_user);
    
    // Open a pipe to sendmail.
    sprintf(buf, "%s '%s'" , SENDMAIL_COMMAND, local_user);
    mailpipe = popen(buf, "w");
    if (mailpipe != NULL)
    {
	int finished = 0;

	// Send the header
	fprintf(mailpipe, "From: %s@%s (%s::%s)\n",
		config_vmsmailuser, config_hostname, remote_hostname,
		remote_user);
	fprintf(mailpipe, "Subject: %s\n", subject);
	fprintf(mailpipe, "To: %s\n", local_user);

	fprintf(mailpipe, "\n");
	
	// Get the text of the message
	do
	{
	    stat = read(sock, buf, sizeof(buf));
	    if (stat == -1 && errno != ENOTCONN)
	    {
		perror("Error reading message text");
		return;
	    }
	    
	    // VMS sends a NUL as the end of the message
	    if (buf[stat-1] == '\0') finished = 1;
	    if (stat > 0)
	    {
		buf[stat] = '\0';
		fprintf(mailpipe, "%s\n", buf);
	    }
	}
	while (stat > 0 && !finished);
	pclose(mailpipe);
    }
    else
    {
	perror("Can't open pipe to sendmail");
    }
    
    // Send acknowledge
    write(sock, "\001\000\000\000", 4);
    close(sock);
}
